using AutoMapper;
using Infrastructure;
using Infrastructure.Entities;
using Infrastructure.Repositories;
using Server.Dto.Request;
using Server.Dto.Response;
using Server.IServices;
using Shared;

namespace Server.Services
{
    public class ExerciceTemplateService : IExerciceTemplateSercice
    {
        private readonly OptifitDbContext _context;
        private readonly IExerciceTemplateRepository _exerciceTemplateRepository;
        private readonly IMapper _mapper;

        public ExerciceTemplateService(OptifitDbContext context, IExerciceTemplateRepository exerciceTemplateRepository, IMapper mapper)
        {
            _context = context;
            _exerciceTemplateRepository = exerciceTemplateRepository;
            _mapper = mapper;
        }

        public async Task<PaginatedResult<ResponseTemplateExerciceDto>> GetExercices(int page, int size, bool ascending = true)
        {
            // Ici, on part du principe que la méthode du repository attend un index de page à partir de 0.
            // Vous pouvez adapter l'appel selon la signature de votre méthode.
            var exercices = await _exerciceTemplateRepository.GetPaginatedListAsync(page - 1, size, null);
            return _mapper.Map<PaginatedResult<ResponseTemplateExerciceDto>>(exercices);
        }

        public async Task<ResponseTemplateExerciceDto?> GetExerciceById(string id)
        {
            var exercice = await _exerciceTemplateRepository.GetByIdAsync(id);
            return exercice == null ? null : _mapper.Map<ResponseTemplateExerciceDto>(exercice);
        }

        public async Task<ResponseTemplateExerciceDto> CreateExercice(RequestTemplateExerciceDto request)
        {
            // On mappe le DTO de requête vers l'entité
            var exercice = _mapper.Map<Exercice_template>(request);
            
            // Comme le DTO ne fournit pas le Name et que celui-ci est requis,
            // vous pouvez ici définir une valeur par défaut ou appliquer une logique de création.
            exercice.Name = string.IsNullOrWhiteSpace(exercice.Name) ? "Exercice Template" : exercice.Name;
            
            await _exerciceTemplateRepository.InsertAsync(exercice);
            await _context.SaveChangesAsync();
            return _mapper.Map<ResponseTemplateExerciceDto>(exercice);
        }

        public async Task<ResponseTemplateExerciceDto?> UpdateExercice(string id, RequestTemplateExerciceDto request)
        {
            var exercice = await _exerciceTemplateRepository.GetByIdAsync(id);
            if (exercice == null)
                return null;

            // Met à jour l'entité en mappant le DTO dans l'entité existante
            _mapper.Map(request, exercice);
            _exerciceTemplateRepository.Update(exercice);
            await _context.SaveChangesAsync();
            return _mapper.Map<ResponseTemplateExerciceDto>(exercice);
        }

        public async Task<bool> DeleteExercice(string id)
        {
            var exercice = await _exerciceTemplateRepository.GetByIdAsync(id);
            if (exercice == null)
                return false;

            _exerciceTemplateRepository.Delete(id);
            await _context.SaveChangesAsync();
            return true;
        }
    }
}